#!/bin/sh
################################################################################
# download-helper.sh                                                           #
#  Given ARM user ID, Order ID and optionally, a download directory, this      #
#  script will monitor for the ordered data to be ready and download the files #
#  automatically into the specified directory, or current directory.           #
#                                                                              #
#  This script is intended to be invoked right after the order is placed, on   #
#  Unix (*BSD, OS X, etc.) or Linux systems.                                   #
#                                                                              #
# February 2018                                                                #
#                                                                              #
################################################################################
#
# Global definitions
VERSION="20200610.0"
#FTPHOST="ret2b.ornl.gov"
FTPHOST="adc-retrieval1.ornl.gov"
ORDSTATURL="https://adc.arm.gov/OrderService/orders/status"
SLEEPTIME=120

#
# Subroutines
#
cleanup()
{
    echo "Cleaning up ..."
    if [ ${tmpdir:-xxx} != "xxx" ]; then
        if [ -d $tmpdir ]; then
            rm -rf $tmpdir 2> /dev/null
        fi
    fi
}

clean_quit()
{
    echo "Interrupted ..."
    cleanup
    echo "Exiting ..."
    exit 1
}

cmd_exists()
{
    cmd=$1
    which $cmd > /dev/null 2>&1
    if [ $? = 1 ]; then
        echo
        echo "ERROR"
        echo "  ${cmd}: command not found!"
        echo 
        echo "Exiting..."
        exit 1
    fi
} 

usage()
{
    echo
    echo "Usage:"
    echo "  $0 <ARM User ID> <Order ID> [<Download directory path>]"
    echo 
}


checkCmd()
{
    cmd=$1
    which $cmd > /dev/null 2>&1
    return $?
}

ftpGetFiles()
{
    local spath="$1"; shift
    local dpath="$1"; shift
    local tmpdir="$1"; shift

    # FTP and check if files exist for download
    echo "Checking to see if files at $FTPHOST:/${spath} ..."

    ftp -p $FTPHOST << ....EOF > ${tmpdir}/files.txt
    ls ${spath}
    exit
....EOF
    
    fcount=$(grep -vE "\.$|\.\.$" ${tmpdir}/files.txt | wc -l)
    if [ $fcount -le 0 ]; then
        echo "No files to download from ${FTPHOST}:/${spath} ..."
        cleanup
        echo "Exiting ..."
        exit 0
    fi
    
    # Download the files
    echo "Downloading files into ${dpath} ..."
    mkdir -p ${dpath}
    cd ${dpath}
    echo
    echo ".................. FTP log follows .................."
    ftp -p -v -i $FTPHOST << ....EOF
    cd ${spath}
    mget *
    exit
....EOF
    echo ".................. End FTP log ......................"

    if [ $? -eq 0 ]; then
        return 0
    else
        return 1
    fi
}


doFTP()
{
    local uid=$1; shift
    local ordid=$1; shift
    local dloaddir=$1; shift

    # Check .netrc. Create and/or populate, if required.
    if [ -f ~/.netrc ]; then
        # Check if entry for default exists
        grep "^default" ~/.netrc > /dev/null 2>&1
        if [ $? != 0 ]; then
            echo "Populating ~/.netrc file ..."
            echo "default login ftp password ftp" >> ~/.netrc
        fi
    else
        echo "Populating ~/.netrc file ..."
        echo "default login ftp password ftp" > ~/.netrc
        chmod 600 ~/.netrc
    fi
    
    # Make tmp dir
    tmpdir="/tmp/arm-download-helper.$$"
    mkdir ${tmpdir}
    
    # Remember original directory
    startdir=$(pwd)

    # Get original order files
    src_path="${uid}/${ordid}"
    dload_path="${dloaddir}/${ordid}"
    ftpGetFiles "$src_path" "$dload_path" "${tmpdir}"

    # Get extracted variables, if any
    cd $startdir
    src_path="${uid}/${ordid}_extractedData"
    dload_path="${dloaddir}/${ordid}_extractedData"
    echo "Checking to see if variable extraction data exists ..."
    ftpGetFiles "$src_path" "$dload_path" "${tmpdir}"

    # Get CSV files, if any
    cd $startdir
    src_path="${uid}/${ordid}_extractedData/ascii-csv"
    dload_path="${dloaddir}/${ordid}_extractedData/ascii-csv"
    echo "Checking to see if CSV converted data exists ..."
    ftpGetFiles "$src_path" "$dload_path" "${tmpdir}"

    # Go back to where we came from
    cd $startdir
    
    # Clean up after ourselves
    cleanup

    return 0
}


doWget()
{
    local uid=$1; shift
    local ordid=$1; shift
    local dloaddir=$1; shift

    startdir=$(pwd)

    # Create destination directory, if required.
    if [ "${dloaddir}" != "." ]; then
        if [ ! -d ${dloaddir} ]; then
            mkdir ${dloaddir}
            if [ $? -ne 0 ]; then
                /bin/echo -n "ERROR - Failed to create missing ${dloaddir}. "
                echo "Using current directory."
            fi
        fi
        echo "Changing to ${dloaddir} directory ..."
        cd ${dloaddir}
    fi

    url="ftp://${FTPHOST}/${uid}/${ordid}"
    echo "Downloading files using wget ..."
    wget -r -nH --cut-dirs=1 --quiet "${url}"
    if [ $? -ne 0 ]; then
        echo "ERROR - Failed to wget ordered files. Please fetch them manually."
    fi

    edurl="ftp://${FTPHOST}/${uid}/${ordid}_extractedData"
    wget -r -nH --cut-dirs=1 --quiet "${url}"
    if [ $? -ne 0 ]; then
        echo "ERROR - Failed to wget extracted data files. Please fetch them manually."
    fi

    # Go back to where we came from
    cd $startdir


    return 0
}


curlGetFiles()
{
    local url="$1"; shift
    local dlpath="$1"; shift

    if [ ! -d "${dlpath}" ]; then
        mkdir -p ${dlpath}
        if [ $? -ne 0 ]; then
            echo "ERROR - Failed to mkdir ${dlpath}"
            return 1
        fi
    fi

    cd ${dlpath}
    echo "Downloading into ${dlpath} ..."

    curl -s "${url}" | grep -v "^dr" | awk '{print $9}' \
      | xargs -t -I % curl -o % -s "${url}"/%
    if [ $? -ne 0 ]; then
        echo "ERROR - Failed to cURL files from ${url}"
        echo "Please fetch them manually."
        return 1
    fi

    return 0
}


doCurl()
{
    local uid=$1; shift
    local ordid=$1; shift
    local dloaddir=$1; shift

    startdir=$(pwd)

    # Create destination directory, if required.
    if [ "${dloaddir}" != "." ]; then
        if [ ! -d ${dloaddir} ]; then
            mkdir -p ${dloaddir}
            if [ $? -ne 0 ]; then
                /bin/echo -n "ERROR - Failed to create missing ${dloaddir}. "
                exit 1
            fi
        fi
        echo "Changing to ${dloaddir} directory ..."
        cd ${dloaddir}
    fi

    url="ftp://${FTPHOST}/${uid}/${ordid}/"
    echo "Downloading files using cURL ..."
    dload_path="${ordid}"
    curlGetFiles "${url}" "${dload_path}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    echo
    echo "Checking to see if order contains extracted variables ..."
    edurl="ftp://${FTPHOST}/${uid}/${ordid}_extractedData/"
    cd ${startdir}
    dload_path="${dloaddir}/${ordid}_extractedData"
    curlGetFiles "${edurl}" "${dload_path}"
    if [ $? -eq 0 ]; then
        echo
    	echo "Checking to see if order requested CSV conversion ..."
    	csvurl="ftp://${FTPHOST}/${uid}/${ordid}_extractedData/ascii-csv/"
        cd ${startdir}
        dload_path="${dloaddir}/${ordid}_extractedData/ascii-csv"
        curlGetFiles "${csvurl}" "${dload_path}"
        if [ $? -ne 0 ]; then
            return 1
        fi
    fi

    # Go back to where we came from
    cd $startdir

    return 0
}


#
# Main
#
trap clean_quit INT KILL TERM

# Check for dependent executables
unalias which 2> /dev/null
cmd_exists which
cmd_exists curl
checkCmd ftp
ftp_chk=$?
checkCmd wget
wget_chk=$?
checkCmd curl
curl_chk=$?
if [ $ftp_chk -a $wget_chk -a $curl_chk ]
then
    echo > /dev/null
else
    echo "Neither ftp, wget nor curl exists!"
    echo "Exiting ..."
    exit 1
fi

if [ $ftp_chk -eq 0 ]; then
    xfercmd="ftp"
elif [ $curl_chk -eq 0 ]; then
    xfercmd="curl"
elif [ $wget_chk -eq 0 ]; then
    xfercmd="wget"
fi

# Check argument count
if [ $# = 2 ]; then
    uid=$1
    ordid=$2
    dloaddir="."
elif [ $# = 3 ]; then
    uid=$1
    ordid=$2
    dloaddir=$3
else
    usage
    exit 1
fi


# Check to see if the order has been serviced
while [ 1 ]; do
    json_str=$(curl -s "${ORDSTATURL}/${ordid}")

    status=$(echo $json_str | \
      tr -d '{}"' | tr ',' '\n'  | grep "^curStep:" | cut -d: -f2)

    if [ "${status}" = "complete" ]; then
        break
    fi

    echo "Order not yet ready for download. Waiting $SLEEPTIME seconds before checking again ..."
    sleep ${SLEEPTIME}
done

if [ "${xfercmd}" = "ftp" ]; then
    doFTP $uid $ordid $dloaddir
elif [ "${xfercmd}" = "curl" ]; then
    doCurl $uid $ordid $dloaddir
elif [ "${xfercmd}" = "wget" ]; then
    doWget $uid $ordid $dloaddir
else
    echo "Nothing to do! ftp nor wget found."
fi

echo "Exiting ..."
exit 0


